/*
 * RSSBot
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.rssbot.jobs

import com.prof18.rssparser.RssParser
import com.prof18.rssparser.model.RssChannel
import dev.inmo.krontab.KronScheduler
import dev.kord.common.entity.Snowflake
import dev.kord.core.behavior.channel.asChannelOf
import dev.kord.core.behavior.channel.createMessage
import dev.kord.core.entity.channel.MessageChannel
import kotlinx.coroutines.Job
import net.moonleay.rssbot.Bot
import net.moonleay.rssbot.data.database.entry.RSSData
import net.moonleay.rssbot.data.database.repository.RSSRepository
import net.moonleay.rssbot.data.database.repository.SubscriptionRepository
import net.moonleay.rssbot.jobs.component.CronjobType
import net.moonleay.rssbot.jobs.component.ICronjob
import net.moonleay.rssbot.util.EmbedUtil
import net.moonleay.rssbot.util.Logger
import net.moonleay.rssbot.util.MessageUtil

object FeedUpdater : ICronjob {
    override val jobName: String
        get() = "FeedUpdater"
    override val jobIncoming: String
        get() = "0 /20 * * * * 0o *" //Every 20 seconds
    override val jobType: CronjobType
        get() = CronjobType.INFINITE
    override val continueJob: Boolean
        get() = true
    override lateinit var cronjobJob: Job
    override lateinit var scheduler: KronScheduler

    override suspend fun jobFunction() {
        Logger.out("Updating feeds...")
        update(true)
    }

    suspend fun update(automated: Boolean = false) {
        Logger.out("Updating feeds [${if (automated) "AUTOMATED" else "MANUALLY"}]...")
        for (data in SubscriptionRepository.getAll()) {
            if (Bot.bot.kordRef.getChannel(Snowflake(data.channelId)) == null) {
                Logger.out("Channel ${data.channelId} is not available anymore ; deleting...")
                SubscriptionRepository.delete(data.channelId, data.feedUrl)
            }
            val parser = RssParser()
            var rss: RssChannel? = null
            runCatching { // this sucks
                rss = parser.getRssChannel(data.feedUrl)
                Logger.out("Updating feed ${data.feedUrl} for ${data.channelId} ...")
            }.onFailure {
                Logger.out("This feed is not valid: ${it.message}; deleting...")
                SubscriptionRepository.delete(data.channelId, data.feedUrl)
            }
            if (rss == null)
                continue
            for (rssData in rss!!.items) {
                if (rssData.guid == null || RSSRepository.exists(RSSData(rssData.guid!!, data.id)))
                    continue
                val channel = Bot.bot.kordRef.getChannel(Snowflake(data.channelId))!!.asChannelOf<MessageChannel>()
                channel.createMessage {
                    this.embeds.add(
                        MessageUtil.getRSSEmbed(
                            data.feedColor,
                            rssData.author ?: "Anonymous",
                            rssData.title ?: "Untitled",
                            rssData.description ?: "No description",
                            rssData.link ?: "https://moonleay.net/",
                            rssData.image ?: "",
                            if (rssData.link != null) EmbedUtil.getSiteLogo(rssData.link!!) else "https://static.moonleay.net/img/no-image.png",
                            data.feedName,
                        )
                    )
                }
                RSSRepository.write(RSSData(rssData.guid!!, data.id))
            }
        }
    }
}
